;
; AutoIt Version: 3.0
; Language:       English
; Platform:       Win9x/NT
; Author:         Michael W. Bayley
;
; Script Function:
;   MultiRun - Run a multiple programs
;
; This program/script assumes it is being run fom the MultiRun directory.
; It also assumes the file "MultiRun.mrn" is in the  same directory.  The
; script will run the programs specified by the profile name supplied on the
; command line.


; Includes
#include	<Constants.au3>
#include	<GUIConstants.au3>

#include	"MultiRunDefs.au3"
#include	"MultiRunFunc.au3"

; Global string values
$TitleBar = $Title & " v" & $ProgramMajor & "." & $ProgramMinor & " - "
$TitleBar &= "(c)2005-2007 " & $Author
$TitleBar2 = $Title & " v" & $ProgramMajor & "." & $ProgramMinor

; Log File Information
$Log = ""

; Ini/Mrn File Information
$IniFile = ""
$Profile = ""
$Items = 0
$Item = ""
$Item2 = ""
$Index = 0
$ProfileSectionName = ""

; Program Info
$SectionID = 0
$ID = ""
$Major = 0
$Minor = 0

; Program Information
$RunProgram = ""
$RunDirectory = ""
$RunParams = ""
$RunProgramMode = 0
$RunSingleInstance = 0
$RunTitleMode = 0
$RunTitle1 = ""
$RunTiTle2 = ""
$RunKeys = ""
$RunWait = 0
$RunTimeout = 0
$RunWidth = 0
$RunHeight = 0
$RunXPos = 0
$RunYPos = 0
$RunError = 0
$RunSearch = ""
$RunPID = -1

; Dialog Values
$MainDlg = -1
$MainList = -1
$MainRun = -1
$MainCancel = -1

$TextProfiles = "Profiles"
$TextRun = "Run Profile"
$TextCancel = "Cancel"

$SepChar = Chr(187)
$List = 0
$Entry = ""
$Count = 0
$Cancelled = 0
$NotDone = 0
$Msg = 0


;
; Main($CmdLine[]) - Run multiple programs
;
; Entry:	    $CmdLine[0]		Number of command line parameters
;		        $CmdLine[1]		Profile name (Defaults to "(Default) if not supplied
; Returns:	    0 if all ok
;		        1 if failure
; Procedure:    1. If no parameters then use (Default) Profile.
;			    2. If one parameter then it is the Profile.
;			    3. If two parameters then 1st=Ini/MrnFile, 2nd=Profile
;			    4. Error if too many parameters.
;			    5. Read the .ini/.mrn file, error if it does not exist.
;			    6. Create log file.
;			    7. I default profile selected and no items then handle menu mode.
;			    8. Process the.ini/.mrn file.
;			    9.  a) Launch the program.
;			        b) Wait for window to open.
;			        c) Position and size the window.
;			        d) Send keys to window.
;			        e) Wait (seconds or program close.
;			    10. Next program (goto 5).
;			    11. Exit all done.
;

AutoItSetOption("RunErrorsFatal", 0)
AutoItSetOption("GUIDataSeparatorChar", $SepChar)
FileChangeDir(@ScriptDir)

; 1. If no parameters then use default profile.
If $CmdLine[0] == 0 Then
	$IniFile = @ScriptDir
	If StringRight($IniFile, 1) == "\" Then
		$IniFile &= $IniFileName
	Else
		$IniFile &= "\" & $IniFileName
	EndIf
	$Profile = $IniDefaultProfile

	
; 2. If one parameter then it is the Profile.
ElseIf $CmdLine[0] == 1 Then
	$IniFile = @ScriptDir
	If StringRight($IniFile, 1) == "\" Then
		$IniFile &= $IniFileName
	Else
		$IniFile &= "\" & $IniFileName
	EndIf
	$Profile = $CmdLine[1]

	
; 3. If two parameters then 1st=Ini/MrnFile, 2nd=Profile
ElseIf $CmdLine[0] == 2 Then
	If StringMid($CmdLine[1], 2, 2) == ":\" Then
		$IniFile = $CmdLine[1]
	ElseIf StringLeft($CmdLine[1], 1) == "\" Then
		$IniFile = StringLeft(@ScriptDir, 2)
		$IniFile &= $CmdLine[1]
	ElseIf StringInStr($CmdLine[1], ".\") Then
		$IniFile = $CmdLine[1]
	Else
		$IniFile = @ScriptDir
		If StringRight($IniFile, 1) == "\" Then
			$IniFile &= $IniFileName
		Else
			$IniFile &= "\" & $IniFileName
		EndIf
	EndIf
	$Profile = $CmdLine[2]

	
; 4. Error if too many parameters.
Else
	$msg = "Error - Too many parameters!" & @CRLF & @CRLF
	$msg &= "Command line syntax = MultiRun.exe <ConfigFile> <ProfileName>" & @CRLF & @CRLF
	$msg &= "If no config file is supplied then the default config file 'MultiRun.mrn'" & @CRLF
	$msg &= "will be used.  If no profile is supplied then the default profile is used" & @CRLF
	$msg &= "'Default'.  If the config file is supplied then a profile must be supplied." & @CRLF & @CRLF
	MsgBox(0, $TitleBar, $msg)
	Exit(1)
EndIf


; 5. Read the .ini/.mrn file, error if it does not exist.
If FileExists($IniFile) == 0 Then
	MsgBox(0, $TitleBar, "Error - Unable to open '" & $IniFile & "'!")
	Exit(1)
EndIf


; 6. Create the log file.
$text = IniRead($IniFile, $IniSection, $IniLog, $Log)
If StringLen($text) Then
	If StringMid($text, 2, 2) == ":\" Then
		$Log = $text
	ElseIf StringLeft($text, 1) == "\" Then
		$Log = StringLeft(@ScriptDir, 2)
		$Log &= $CmdLine[1]
	ElseIf StringInStr($text, ".\") Then
		$Log = $text
	Else
		$Log = @ScriptDir
		If StringRight($Log, 1) == "\" Then
			$Log &= $text
		Else
			$Log &= "\" & $text
		EndIf
	EndIf
EndIf
FileDelete($Log)


; 7. I default profile selected and no items then handle menu mode.
If $Profile == $IniDefaultProfile Then
	$Items = Number(IniRead($IniFile, $Profile, $IniItems, 0))
	If $Items < 1 Then
		PostLog($Log, "Entering menu mode...", 0)

		; Process Menu Mode
		$MainDialog = GUICreate($TitleBar2, 280, 275)
		GUICtrlCreateLabel($TextProfiles, 20, 10, 240, 20)
		$MainList = GUICtrlCreateList("", 20, 30, 240, 180)
		$MainRun = GUICtrlCreateButton($TextRun, 50, 225, 80, 30)
		$MainCancel = GUICtrlCreateButton($TextCancel, 150, 225, 80, 30)

		; Populate dialog
		$Items = Number(IniRead($IniFile, $IniProfilesSection, $IniItems, 0))
		$Entry = ""
		$Index = 1
		$Count = 0
		$List = ""
		While $Index <= $Items
			$Item = IniRead($IniFile, $IniProfilesSection, $IniItem & $Index, "")
			If StringLen($Item) AND $Item <> $IniDefaultProfile Then
				$Item2 = Number(IniRead($IniFile, $Item, $IniItems, 0))
				If $Item2 Then
					$List &= $SepChar & $Item
					If StringLen($Entry) == 0 Then
						$Entry = $Item
					ElseIf $Item < $Entry Then
						$Entry = $Item
					EndIf
					$Count += 1
				EndIf
			EndIf
			$Index += 1
		WEnd
		
		; Abort if no items to select from
		If $Count == 0 Then
			PostLog($Log, "Error - No empty profiles available.", 1)
			Exit(1)
		EndIf

		GUICtrlSetData($MainList, $List, $Entry)
		GUICtrlSetState($MainRun, $GUI_DEFBUTTON)
		GUISetState(@SW_SHOW, $MainDialog)

		;  Process user commands..
		$Cancelled = 0
		$NotDone = 1
		While $NotDone
			$Msg = GUIGetMsg()

			Select
				Case $Msg == $GUI_EVENT_CLOSE
					; Close dialog
					$Cancelled = 1
					$NotDone = 0
				Case $Msg == $MainRun
					; Run button
					$NotDone = 0
				Case $Msg == $MainCancel
					; Cancel button
					$Cancelled = 1
					$NotDone = 0
			EndSelect
		WEnd

		; Setup profile.
		If $Cancelled Then
			GUIDelete($MainDialog)
			Exit(0)
		EndIf

		$Profile = GUICtrlRead($MainList)
		GUIDelete($MainDialog)
	EndIf
EndIf


; 8. Process the program .ini/.mrn file
PostLog($Log, "Processing config file '" & $IniFile & "'...", 0)
$SectionID = Number(IniRead($IniFile, $IniSection, $IniSectionID, 0))
$ID = IniRead($IniFile, $IniSection, $IniProgramID, "")
$Major = Number(IniRead($IniFile, $IniSection, $IniMajor, 0))
$Minor = Number(IniRead($IniFile, $IniSection, $IniMinor, 0))

; Proper section ID
If $SectionID <> $IniMainSectionID Then
	PostLog($Log, "Error - Invalid section ID in main section.", 1)
	Exit(1)
EndIf

; Proper Program ID
If $ID <> $ProgramID Then
	PostLog($Log, "Error - Invalid program ID in main section.", 1)
	Exit(1)
EndIf

; Proper Version
If $Major <> $ProgramMajor OR $Minor <> $ProgramMinor Then
	PostLog($Log, "Error - Incompatible version of '" $IniFile & "'.", 1)
	Exit(1)
EndIf
PostLog($Log, $ID & " version " & $Major & "." & $Minor, 0)

; Profile Section Exists
; Profile Section ID
$SectionID = Number(IniRead($IniFile, $Profile, $IniSectionID, 0))
If $SectionID <> $IniProfileEntryID Then
	PostLog($Log, "Error - Invalid section ID or profile '" & $Profile & "' does not exist.", 1)
	Exit(1)
EndIf

; Profile Item Count
$Items = Number(IniRead($IniFile, $Profile, $IniItems, 0))
If $Items < 1 Then
	PostLog($Log, "Error - Invalid section ID or profile '" & $Profile & "' does not exist.", 1)
	Exit(1)
EndIf
PostLog($Log, "Profile '" & $Profile & "'", 0)


For $Index = 1 To $Items
	$RunError = 0
	$ProfileSectionName = $Profile & "-" & StringRight("00" & $Index, 2)
	PostLog($Log, "Processing program section '" & $ProfileSectionName & "'", 0)
	

	$SectionID = Number(IniRead($IniFile, $ProfileSectionName, $IniSectionID, 0))
	If $SectionID <> $IniRunSectionID Then
		PostLog($Log, "  Error - Invalid section ID or missing program entry '" & $ProfileSectionName & "' does not exist.", 1)
		RunError = 1
	Else
		$text = IniRead($IniFile, $ProfileSectionName, $IniRunProgram, "")
		If StringLen($text) == 0 Then
			PostLog($Log, "  Error - NULL program entry in section '" & $ProfileSectionName & "' does not exist.", 1)
			RunError = 1
		Else
			PostLog($Log, "  Program name: " & $text, 0)
			If StringMid($text, 2, 2) == ":\" Then
		$RunProgram = $text
			ElseIf StringLeft($text, 1) == "\" Then
				$RunProgram = StringLeft(@ScriptDir, 2)
				$RunProgram &= $CmdLine[1]
			ElseIf StringInStr($text, ".\") Then
				$RunProgram = $text
			Else
				$RunProgram = @ScriptDir
				If StringRight($RunProgram, 1) == "\" Then
					$RunProgram &= $text
				Else
					$RunProgram &= "\" & $text
				EndIf
			EndIf
			PostLog($Log, "  Adjusted program name: " & $RunProgram, 0)

			$text = IniRead($IniFile, $ProfileSectionName, $IniRunDir, "")
			PostLog($Log, "  Start directory: " & $text, 0)
			If StringMid($text, 2, 2) == ":\" Then
				$RunDirectory = $text
			ElseIf StringLeft($text, 1) == "\" Then
				$RunDirectory = StringLeft(@ScriptDir, 2)
				$RunDirectory &= $CmdLine[1]
			ElseIf StringInStr($text, ".\") Then
				$RunDirectory = $text
			Else
				$RunDirectory = @ScriptDir
				If StringRight($RunDirectory, 1) == "\" Then
					$RunDirectory &= $text
				Else
					$RunDirectory &= "\" & $text
				EndIf
			EndIf
			PostLog($Log, "  Adjusted start directory: " & $RunDirectory, 0)

			$RunParams = IniRead($IniFile, $ProfileSectionName, $IniRunParams, "")
			PostLog($Log, "  Program parameters: " & $RunParams, 0)
		EndIf
	EndIf

	If $RunError == 0 Then
		$RunProgramMode = Number(IniRead($IniFile, $ProfileSectionName, $IniRunProgramMode, 0))
		If $RunProgramMode < $IniRunProgramModeNormal OR $RunProgramMode > $IniRunProgramModeHide Then
			PostLog($Log, "  Error - Invalid program run mode in section '" & $ProfileSectionName & "' does not exist.", 1)
			RunError = 1
		Else
			Select
				Case $RunProgramMode == $IniRunProgramModeNormal
					PostLog($Log, "  Run mode: Normal window", 0)
					$RunProgramMode = ""
				Case $RunProgramMode == $IniRunProgramModeMax
					PostLog($Log, "  Run mode: Maximized", 0)
					$RunProgramMode = @SW_MAXIMIZE
				Case $RunProgramMode == $IniRunProgramModeMin
					PostLog($Log, "  Run mode: Minimized", 0)
					$RunProgramMode = @SW_MINIMIZE
				Case $RunProgramMode == $IniRunProgramModeHide
					PostLog($Log, "  Run mode: Hidden", 0)
					$RunProgramMode = @SW_HIDE
			EndSelect

			$RunSingleInstance = Number(IniRead($IniFile, $ProfileSectionName, $IniRunSingleInstance, 0))
			If $RunSingleInstance Then
				PostLog($Log, "  Run single instance: TRUE", 0)
			Else
				PostLog($Log, "  Run single instance: FALSE", 0)
			EndIf

			$RunTitleMode = Number(IniRead($IniFile, $ProfileSectionName, $IniRunTitleMode, 0))
			If $RunTitleMode < $IniRunTitleModeNone OR $RunTitleMode > $IniRunTitleModeRegexp Then
				PostLog($Log, "  Error - Invalid title match mode in section '" & $ProfileSectionName & "' does not exist.", 1)
				$RunTitleMode = $IniRunTitleModeNone
			Else
				If $RunTitleMode == $IniRunTitleModeNone Then
					PostLog($Log, "  Title match mode: Disabled", 0)
					AutoItSetOption("WinTitleMatchMode", 1)
				Else
					Select
						Case $RunTitleMode == $IniRunTitleModeExact
							PostLog($Log, "  Title match mode: Exact", 0)
							AutoItSetOption("WinTitleMatchMode", 3)
						Case $RunTitleMode == $IniRunTitleModeStart
							PostLog($Log, "  Title match mode: From start", 0)
							AutoItSetOption("WinTitleMatchMode", 1)
						Case $RunTitleMode == $IniRunTitleModeContains
							PostLog($Log, "  Title match mode: Contains", 0)
							AutoItSetOption("WinTitleMatchMode", 2)
						Case $RunTitleMode == $IniRunTitleModeExact
							PostLog($Log, "  Title match mode: From start and at end", 0)
							AutoItSetOption("WinTitleMatchMode", 4)
					EndSelect

					$RunTitle1 = IniRead($IniFile, $ProfileSectionName, $IniRunTitle1, "")
					PostLog($Log, "  Title 1: '" & $RunTitle1 & "'", 0)
					$RunTitle2 = IniRead($IniFile, $ProfileSectionName, $IniRunTitle2, "")
					PostLog($Log, "  Title 2: '" & $RunTitle2 & "'", 0)
					If StringLen($RunTitle1) == 0 AND StringLen($RunTitle2) == 0 Then
						PostLog($Log, "  Error - NULL title in section '" & $ProfileSectionName & "' does not exist.", 1)
					Else
						$RunKeys = IniRead($IniFile, $ProfileSectionName, $IniRunKeys, "")
						PostLog($Log, "  Keys to send to program: '" & $RunKeys & "'", 0)

						$RunWait = Number(IniRead($IniFile, $ProfileSectionName, $IniRunWait, 0))
						If $RunWait == $IniRunWaitClose Then
							PostLog($Log, "  Wait: Until program closes", 0)
						Else
							PostLog($Log, "  Wait: " & $RunWait & "seconds after start", 0)
						EndIf

						$RunTimeout = Number(IniRead($IniFile, $ProfileSectionName, $IniRunTimeout, 0))
						If $RunTimeout == $IniRunTimeoutForever Then
							PostLog($Log, "  Timeout: Wait forever", 0)
						Else
							PostLog($Log, "  Timeout: " & $RunWait & "seconds after start", 0)
						EndIf

						$RunWidth = Number(IniRead($IniFile, $ProfileSectionName, $IniRunWidth, -1))
						If $RunWidth == $IniRunCenter Then
							PostLog($Log, "  Window width: Centered", 0)
						Else
							PostLog($Log, "  Window width: " & $RunWidth, 0)
						EndIf

						$RunHeight = Number(IniRead($IniFile, $ProfileSectionName, $IniRunHeight, -1))
						If $RunHeight == $IniRunCenter Then
							PostLog($Log, "  Window height: Centered", 0)
						Else
							PostLog($Log, "  Window height: " & $RunHeight, 0)
						EndIf

						$RunXPos = Number(IniRead($IniFile, $ProfileSectionName, $IniRunXPos, -1))
						If $RunXPos == $IniRunCenter Then
							PostLog($Log, "  Window X position: Centered", 0)
						Else
							PostLog($Log, "  Window X position: " & $RunXPos, 0)
						EndIf

						$RunYPos = Number(IniRead($IniFile, $ProfileSectionName, $IniRunYPos, -1))
						If $RunYPos == $IniRunCenter Then
							PostLog($Log, "  Window Y position: Centered", 0)
						Else
							PostLog($Log, "  Window Y position: " & $RunYPos, 0)
						EndIf
					EndIf
				EndIf
			EndIf
		EndIf
	EndIf


	; 9a. Launch the program
	If $RunError == 0 Then
		PostLog($Log, "  Running program '" & $RunProgram & "'", 0)
		$RunPID = LaunchProgram($RunProgram, $RunDirectory, $RunParams, $RunSingleInstance, $RunProgramMode)
		If @error Then
			Select
				Case @error == 1
					PostLog($Log, "  Error - Unable to locate program '" & $RunProgram & "'", 1)
					$RunError = 1
				Case @error == 2
					PostLog($Log, "  Error - Unable to locate directory '" & $RunDirectory & "'", 1)
					$RunError = 1
				Case @error == 3
					PostLog($Log, "  Error - '" & $RunDirectory & "' is not a directory", 1)
					$RunError = 1
				Case @error == 4
					PostLog($Log, "  Error - Unable to run program '" & $RunProgram & "'", 1)
					$RunError = 1
				Case @error == -1
					PostLog($log, "  Program already running, launch skipped", 0)
			EndSelect
		EndIf
	EndIf


	;  9b) Wait for window to open.
	If $RunError == 0 Then
		PostLog($Log, "  Waiting for program window", 0)
		$RunSearch = WaitProgramWindow($RunProgram, $RunTitleMode, $RunTitle1, $RunTitle2)
		If @error Then
			Select
				Case @error == 1
					PostLog($Log, "  Error - Program '" & $RunProgram & "' is no longer running", 1)
				Case @error == 2
					PostLog($Log, "  Error - Invalid title text", 1)
				Case @error == 3
					PostLog($Log, "  Error - Unable to locate window for program '" & $RunProgram & "'", 1)
			EndSelect
			$RunError = 1
		EndIf
	EndIf


	; 9c) Position and size the window.
	If $RunError == 0 Then
		If StringLen($RunSearch) Then
			PostLog($Log, "  Positioning program window", 0)
			If PositionProgramWindow($RunSearch, $RunWidth, $RunHeight, $RunXPos, $RunYPos) == 0 Then
				PostLog($Log, "  Error - Unable to locate window for program '" & $RunProgram & "' is not a directory", 0)
			EndIf
		EndIf


		; 9d) Send keys to window.
		If StringLen($RunKeys) Then
			PostLog($Log, "  Sending keystrokes to program", 0)
			If SendProgramKeys($RunSearch, $RunKeys) == 0 Then
				PostLog($Log, "  Error - Unable to locate window for program '" & $RunProgram & "' is not a directory", 0)
			EndIf
		EndIf


		; 9e) Wait (seconds or program close.
		PostLog($Log, "  Waiting before next program", 0)
		If WaitProgram($RunProgram, $RunWait, $RunTimeout) == 0 Then
			PostLog($Log, "  Error - Timeout waiting for program '" & $RunProgram & "' to terminate", 0)
		EndIf
	EndIf


;10. Next program.
Next


; 11. Exit all done.
PostLog($Log, "Process complete", 0)
Exit(0)


;
; User defined functions
; Logging Functions
;

;
; PostLog($logFile, $logData, $logDialog)
; Entry:		$logFile		File to post to
;				No logging if NULL
;			    $logData		Text to write to log file
;			    $logDialog		Report message via dialog
; Return:		<None>
; Procedure:	1. Skip if not logging.
;			    2. Open the log, report if error.
;			    3. Post data to log;
;			    4. Display dialog to user if desired.
;
Func PostLog($logFile, $logData, $logDialog)
	Local $handle
	Local $timestamp


	; 1. Skip if not logging.
	If StringLen($logFile) Then
		; 2. Open the log, report if error.
		$handle = FileOpen($logFile, 1)
		If $handle == -1 Then
			MsgBox(0, $TitleBar, "Error - Cannot open/create log " & $logFile)
		Else
			; 3. Post data to log.
			$timestamp = @MON & "/" & @MDAY & "/" & @YEAR & " " & @HOUR & ":" & @MIN & ":" & @SEC
			If FileWriteLine($handle, $timestamp & " - " & $Title & ": " & $LogData) == 0 Then
				MsgBox(0, $TitleBar, "Error - Write error in log " & $logFile)
			EndIf
			FileClose($handle)
		EndIf
	EndIf


	; 4. Display dialog to user if desired.
	If $logDialog <> 0 Then
		MsgBox(0, $TitleBar, $logData, 10)
	EndIf
	Return
EndFunc
